@@ -14,6 +14,7 @@ import pytest
 import sys
 
 from google.protobuf.descriptor import FieldDescriptor
+from google.protobuf.internal import api_implementation
 from google.protobuf.message import Message
 
 import testproto.test_pb2 as test_pb2
@@ -43,6 +44,7 @@ from testproto.test3_pb2 import (
 from testproto.test_extensions3_pb2 import (
     MessageOptionsTestMsg,
     scalar_option,
+    SCALAR_OPTION_FIELD_NUMBER,
     repeated_scalar_option,
     enum_option,
     repeated_enum_option,
@@ -60,6 +62,11 @@ from typing import (
     Type,
 )
 
+# C++ python API implementation has some semantic differences from pure python
+# We mainly focus on testing the C++ impl - but just have some flags here
+# to flag off tests that only work on cpp impl
+CPP_IMPL = api_implementation.Type() == "cpp"
+
 UserId = NewType("UserId", int)
 
 
@@ -182,7 +189,8 @@ def test_enum() -> None:
     assert OuterEnum.Name(e3) == "BAR"
 
     # Protobuf itself allows both str and bytes here.
-    assert OuterEnum.Value("BAR") == OuterEnum.Value(b"BAR")
+    if CPP_IMPL:
+        assert OuterEnum.Value("BAR") == OuterEnum.Value(b"BAR")
 
 
 def test_enum_naming_conflicts() -> None:
@@ -204,22 +212,23 @@ def test_has_field_proto2() -> None:
 
     # Erase the types to verify that incorrect inputs fail at runtime
     # Each test here should be duplicated in test_negative to ensure mypy fails it too
-    s_untyped: Any = s
-    with pytest.raises(
-        ValueError, match="Protocol message Simple1 has no field garbage."
-    ):
-        s_untyped.HasField("garbage")
-    with pytest.raises(
-        ValueError,
-        match='Protocol message Simple1 has no singular "a_repeated_string" field',
-    ):
-        s_untyped.HasField("a_repeated_string")
-    with pytest.raises(TypeError, match="bad argument type for built-in operation"):
-        s_untyped.HasField(b"a_string")
+    if CPP_IMPL:
+        s_untyped: Any = s
+        with pytest.raises(
+            ValueError, match="Protocol message Simple1 has no field garbage."
+        ):
+            s_untyped.HasField("garbage")
+        with pytest.raises(
+            ValueError,
+            match='Protocol message Simple1 has no singular "a_repeated_string" field',
+        ):
+            s_untyped.HasField("a_repeated_string")
+        with pytest.raises(TypeError, match="bad argument type for built-in operation"):
+            s_untyped.HasField(b"a_string")
 
-    none_err = "bad argument type for built-in operation"
-    with pytest.raises(TypeError, match=none_err):
-        s_untyped.HasField(None)
+        none_err = "bad argument type for built-in operation"
+        with pytest.raises(TypeError, match=none_err):
+            s_untyped.HasField(None)
 
 
 def test_has_field_proto3() -> None:
@@ -243,32 +252,33 @@ def test_has_field_proto3() -> None:
 
     # Erase the types to verify that incorrect inputs fail at runtime
     # Each test here should be duplicated in test_negative to ensure mypy fails it too
-    s_untyped: Any = s
-    with pytest.raises(
-        ValueError, match="Protocol message SimpleProto3 has no field garbage."
-    ):
-        s_untyped.HasField("garbage")
-    with pytest.raises(
-        ValueError,
-        match='Can\'t test non-optional, non-submessage field "SimpleProto3.a_string" for presence in proto3.',
-    ):
-        s_untyped.HasField("a_string")
-    with pytest.raises(
-        ValueError,
-        match='Can\'t test non-optional, non-submessage field "SimpleProto3.a_outer_enum" for presence in proto3.',
-    ):
-        s_untyped.HasField("a_outer_enum")
-    with pytest.raises(
-        ValueError,
-        match='Protocol message SimpleProto3 has no singular "a_repeated_string" field',
-    ):
-        s_untyped.HasField("a_repeated_string")
-    with pytest.raises(TypeError, match="bad argument type for built-in operation"):
-        s_untyped.HasField(b"outer_message")
+    if CPP_IMPL:
+        s_untyped: Any = s
+        with pytest.raises(
+            ValueError, match="Protocol message SimpleProto3 has no field garbage."
+        ):
+            s_untyped.HasField("garbage")
+        with pytest.raises(
+            ValueError,
+            match='Can\'t test non-optional, non-submessage field "SimpleProto3.a_string" for presence in proto3.',
+        ):
+            s_untyped.HasField("a_string")
+        with pytest.raises(
+            ValueError,
+            match='Can\'t test non-optional, non-submessage field "SimpleProto3.a_outer_enum" for presence in proto3.',
+        ):
+            s_untyped.HasField("a_outer_enum")
+        with pytest.raises(
+            ValueError,
+            match='Protocol message SimpleProto3 has no singular "a_repeated_string" field',
+        ):
+            s_untyped.HasField("a_repeated_string")
+        with pytest.raises(TypeError, match="bad argument type for built-in operation"):
+            s_untyped.HasField(b"outer_message")
 
-    none_err = "bad argument type for built-in operation"
-    with pytest.raises(TypeError, match=none_err):
-        s_untyped.HasField(None)
+        none_err = "bad argument type for built-in operation"
+        with pytest.raises(TypeError, match=none_err):
+            s_untyped.HasField(None)
 
 
 def test_clear_field_proto2() -> None:
@@ -281,13 +291,16 @@ def test_clear_field_proto2() -> None:
     s.ClearField("a_inner")
     s.ClearField("a_repeated_string")
     s.ClearField("a_oneof")
-    s.ClearField(b"a_string")
+    if CPP_IMPL:
+        s.ClearField(b"a_string")
 
-    # Erase the types to verify that incorrect inputs fail at runtime
-    # Each test here should be duplicated in test_negative to ensure mypy fails it too
-    s_untyped: Any = s
-    with pytest.raises(ValueError, match='Protocol message has no "garbage" field.'):
-        s_untyped.ClearField("garbage")
+        # Erase the types to verify that incorrect inputs fail at runtime
+        # Each test here should be duplicated in test_negative to ensure mypy fails it too
+        s_untyped: Any = s
+        with pytest.raises(
+            ValueError, match='Protocol message has no "garbage" field.'
+        ):
+            s_untyped.ClearField("garbage")
 
 
 def test_clear_field_proto3() -> None:
@@ -301,7 +314,8 @@ def test_clear_field_proto3() -> None:
     s.ClearField("outer_message")
     s.ClearField("a_repeated_string")
     s.ClearField("a_oneof")
-    s.ClearField(b"a_string")
+    if CPP_IMPL:
+        s.ClearField(b"a_string")
     s.ClearField("an_optional_string")
     # synthetic oneof from optional field
     s.ClearField("_an_optional_string")
@@ -309,8 +323,11 @@ def test_clear_field_proto3() -> None:
     # Erase the types to verify that incorrect inputs fail at runtime
     # Each test here should be duplicated in test_negative to ensure mypy fails it too
     s_untyped: Any = s
-    with pytest.raises(ValueError, match='Protocol message has no "garbage" field.'):
-        s_untyped.ClearField("garbage")
+    if CPP_IMPL:
+        with pytest.raises(
+            ValueError, match='Protocol message has no "garbage" field.'
+        ):
+            s_untyped.ClearField("garbage")
 
 
 def test_which_oneof_proto2() -> None:
@@ -319,7 +336,8 @@ def test_which_oneof_proto2() -> None:
     assert s.WhichOneof("a_oneof") is None
     s.a_oneof_1 = "hello"
     assert s.WhichOneof("a_oneof") == "a_oneof_1"
-    assert s.WhichOneof(b"a_oneof") == "a_oneof_1"
+    if CPP_IMPL:
+        assert s.WhichOneof(b"a_oneof") == "a_oneof_1"
     assert type(s.WhichOneof("a_oneof")) == str
     field = s.WhichOneof("a_oneof")
     assert field is not None
@@ -341,7 +359,8 @@ def test_which_oneof_proto3() -> None:
     s.a_oneof_1 = "hello"
     s.b_oneof_1 = "world"
     assert s.WhichOneof("a_oneof") == "a_oneof_1"
-    assert s.WhichOneof(b"a_oneof") == "a_oneof_1"
+    if CPP_IMPL:
+        assert s.WhichOneof(b"a_oneof") == "a_oneof_1"
     assert type(s.WhichOneof("a_oneof")) == str
 
     field_a = s.WhichOneof("a_oneof")
@@ -373,6 +392,7 @@ def test_extensions_proto2() -> None:
     s2 = Simple2()
 
     assert isinstance(Extensions1.ext, FieldDescriptor)
+    assert test_pb2.Extensions1.EXT_FIELD_NUMBER == 1000
     assert isinstance(Extensions2.foo, FieldDescriptor)
     assert isinstance(SeparateFileExtension.ext, FieldDescriptor)
 
@@ -412,6 +432,7 @@ def test_extensions_proto3() -> None:
         MessageOptionsTestMsg.DESCRIPTOR.GetOptions().Extensions[scalar_option]
         == "Hello world!"
     )
+    assert SCALAR_OPTION_FIELD_NUMBER == 51234
     assert MessageOptionsTestMsg.DESCRIPTOR.GetOptions().Extensions[
         repeated_scalar_option
     ] == ["A", "B", "C"]
